searchData={"items":[{"type":"module","title":"Rewrite","doc":"`Rewrite` provides a struct that contains all resources that could be handeld\nby `Rewrite`.","ref":"Rewrite.html"},{"type":"function","title":"Rewrite.count/2","doc":"Counts the sources with the given `extname` in the `rewrite` project.","ref":"Rewrite.html#count/2"},{"type":"function","title":"Rewrite.delete/2","doc":"Deletes the source for the given `path` from the `rewrite`. The file on disk\nis not removed.\n\nIf the source is not part of the `rewrite` project the unchanged `rewrite` is\nreturned.","ref":"Rewrite.html#delete/2"},{"type":"function","title":"Examples - Rewrite.delete/2","doc":"iex> {:ok, project} = Rewrite.from_sources([\n    ...>   Source.from_string(\":a\", \"a.exs\"),\n    ...>   Source.from_string(\":b\", \"b.exs\"),\n    ...>   Source.from_string(\":a\", \"c.exs\")\n    ...> ])\n    iex> Rewrite.paths(project)\n    [\"a.exs\", \"b.exs\", \"c.exs\"]\n    iex> project = Rewrite.delete(project, \"a.exs\")\n    iex> Rewrite.paths(project)\n    [\"b.exs\", \"c.exs\"]\n    iex> project = Rewrite.delete(project, \"b.exs\")\n    iex> Rewrite.paths(project)\n    [\"c.exs\"]\n    iex> project = Rewrite.delete(project, \"b.exs\")\n    iex> Rewrite.paths(project)\n    [\"c.exs\"]","ref":"Rewrite.html#delete/2-examples"},{"type":"function","title":"Rewrite.drop/2","doc":"Drops the sources with the given `paths` from the `rewrite` project.\n\nThe files for the dropped sources are not removed from disk.\n\nIf `paths` contains paths that are not in `rewrite`, they're simply ignored.","ref":"Rewrite.html#drop/2"},{"type":"function","title":"Examples - Rewrite.drop/2","doc":"iex> {:ok, project} = Rewrite.from_sources([\n    ...>   Source.from_string(\":a\", \"a.exs\"),\n    ...>   Source.from_string(\":b\", \"b.exs\"),\n    ...>   Source.from_string(\":a\", \"c.exs\")\n    ...> ])\n    iex> project = Rewrite.drop(project, [\"a.exs\", \"b.exs\", \"z.exs\"])\n    iex> Rewrite.paths(project)\n    [\"c.exs\"]","ref":"Rewrite.html#drop/2-examples"},{"type":"function","title":"Rewrite.from_sources/2","doc":"Creates a `%Rewrite{}` from the given sources.\n\nReturns `{:ok, rewrite}` for a list of regular sources.\n\nReturns `{:error, error}` for sources with a missing path and/or duplicated\npaths.","ref":"Rewrite.html#from_sources/2"},{"type":"function","title":"Rewrite.from_sources!/2","doc":"Same as `from_sources/2`, but raises a `Rewrite.Error` exception in case of\nfailure.","ref":"Rewrite.html#from_sources!/2"},{"type":"function","title":"Rewrite.has_source?/2","doc":"Returns `true` when the `%Rewrite{}` contains a `%Source{}` with the given\n`path`.","ref":"Rewrite.html#has_source?/2"},{"type":"function","title":"Examples - Rewrite.has_source?/2","doc":"iex> {:ok, project} = Rewrite.from_sources([\n    ...>   Source.from_string(\":a\", \"a.exs\")\n    ...> ])\n    iex> Rewrite.has_source?(project, \"a.exs\")\n    true\n    iex> Rewrite.has_source?(project, \"b.exs\")\n    false","ref":"Rewrite.html#has_source?/2-examples"},{"type":"function","title":"Rewrite.issues?/1","doc":"Returns `true` if any source has one or more issues.","ref":"Rewrite.html#issues?/1"},{"type":"function","title":"Rewrite.map/2","doc":"Invokes `fun` for each `source` in the `rewrite` project and updates the\n`rewirte` project with the result of `fun`.\n\nReturns a `{:ok, rewrite}` if any update is successful.\n\nReturns `{:error, errors, rewrite}` where `rewrite` is updated for all sources\nthat are updated successful. The `errors` are the `errors` of `update/3`.","ref":"Rewrite.html#map/2"},{"type":"function","title":"Rewrite.map!/2","doc":"Return a `rewrite` project where each `source` is the result of invoking\n`fun` on each `source` of the given `rewrite` project.","ref":"Rewrite.html#map!/2"},{"type":"function","title":"Rewrite.new/1","doc":"Creates an empty project.\n\nThe optional argument is a list of modules implementing the behavior\n`Rewrite.Filetye`. This list is used to add the `filetype` to the `sources` of\nthe corresponding files. The list can contain modules representing a file\ntype or a tuple of `{module(), keyword()}`. Rewrite uses the keyword list from\nthe tuple as the options argument when a file is reading.","ref":"Rewrite.html#new/1"},{"type":"function","title":"Examples - Rewrite.new/1","doc":"iex> project = Rewrite.new()\n    %Rewrite{\n      sources: %{},\n      extensions: %{\n        \"default\" => Source,\n        \".ex\" => Source.Ex,\n        \".exs\" => Source.Ex,\n      }\n    }\n    iex> path = \"test/fixtures/source/hello.txt\"\n    iex> project = Rewrite.read!(project, path)\n    iex> project |> Rewrite.source!(path) |> Source.get(:content)\n    \"hello\\n\"\n    iex> project |> Rewrite.source!(path) |> Source.owner()\n    Rewrite\n\n    iex> project = Rewrite.new([{Rewrite.Source, owner: MyApp}])\n    %Rewrite{\n      sources: %{},\n      extensions: %{\n        \"default\" => {Source, owner: MyApp}\n      }\n    }\n    iex> path = \"test/fixtures/source/hello.txt\"\n    iex> project = Rewrite.read!(project, path)\n    iex> project |> Rewrite.source!(path) |> Source.owner()\n    MyApp","ref":"Rewrite.html#new/1-examples"},{"type":"function","title":"Rewrite.new!/2","doc":"Creates a `%Rewrite{}` from the given `inputs`.\n\nThe optional second argument is a list of modules implementing the behavior\n`Rewrite.Filetye`. For more info, see `new/1`.","ref":"Rewrite.html#new!/2"},{"type":"function","title":"Rewrite.paths/1","doc":"Returns a sorted list of all paths in the `rewrite` project.","ref":"Rewrite.html#paths/1"},{"type":"function","title":"Rewrite.put/2","doc":"Puts the given `source` to the given `rewrite` project.\n\nReturns `{:ok, rewrite}` if successful, `{:error, reason}` otherwise.","ref":"Rewrite.html#put/2"},{"type":"function","title":"Examples - Rewrite.put/2","doc":"iex> project = Rewrite.new()\n    iex> {:ok, project} = Rewrite.put(project, Source.from_string(\":a\", \"a.exs\"))\n    iex> map_size(project.sources)\n    1\n    iex> Rewrite.put(project, Source.from_string(\":b\"))\n    {:error, %Rewrite.Error{reason: :nopath}}\n    iex> Rewrite.put(project, Source.from_string(\":a\", \"a.exs\"))\n    {:error, %Rewrite.Error{reason: :overwrites, path: \"a.exs\"}}","ref":"Rewrite.html#put/2-examples"},{"type":"function","title":"Rewrite.put!/2","doc":"Same as `put/2`, but raises a `Rewrite.Error` exception in case of failure.","ref":"Rewrite.html#put!/2"},{"type":"function","title":"Rewrite.read!/3","doc":"Reads the given `input`/`inputs` and adds the source/sources to the `project`\nwhen not already readed.","ref":"Rewrite.html#read!/3"},{"type":"function","title":"Options - Rewrite.read!/3","doc":"+ `:force`, default: `false` - forces the reading of sources. With\n  `force: true` updates and issues for an already existing source are deleted.","ref":"Rewrite.html#read!/3-options"},{"type":"function","title":"Rewrite.rm/2","doc":"Tries to delete the `source` file and removes the `source` from the `rewrite`\nproject.\n\nReturns `{:ok, rewrite}` if successful, or `{:error, error}` if an error\noccurs.\n\nNote the file is deleted even if in read-only mode.","ref":"Rewrite.html#rm/2"},{"type":"function","title":"Rewrite.rm!/2","doc":"Same as `source/2`, but raises a `Rewrite.Error` exception in case of failure.","ref":"Rewrite.html#rm!/2"},{"type":"function","title":"Rewrite.source/2","doc":"Returns the `%Rewrite.Source{}` for the given `path`.\n\nReturns an `:ok` tuple with the found source, if not exactly one source is\navailable an `:error` is returned.\n\nSee also `sources/2` to get a list of sources for a given `path`.","ref":"Rewrite.html#source/2"},{"type":"function","title":"Rewrite.source!/2","doc":"Same as `source/2`, but raises a `Rewrite.Error` exception in case of\nfailure.","ref":"Rewrite.html#source!/2"},{"type":"function","title":"Rewrite.sources/1","doc":"Returns all sources sorted by path.","ref":"Rewrite.html#sources/1"},{"type":"function","title":"Rewrite.update/2","doc":"Updates the given `source` in the `rewrite` project.\n\nThis function will be usually used if the `path` for the `source` has not\nchanged.\n\nReturns `{:ok, rewrite}` if successful, `{:error, error}` otherwise.","ref":"Rewrite.html#update/2"},{"type":"function","title":"Rewrite.update/3","doc":"Updates a source for the given `path` in the `rewrite` project.\n\nIf `source` a `Rewrite.Source` struct the struct is used to update the\n`rewrite` project.\n\nIf `source` a function the source for the given `path` is passed to the\nfunction and the result is used to update the `rewrite` project.\n\nReturns `{:ok, rewrite}` if the update was successful, `{:error, error}`\notherwise.","ref":"Rewrite.html#update/3"},{"type":"function","title":"Examples - Rewrite.update/3","doc":"iex> a = Source.Ex.from_string(\":a\", \"a.exs\")\n    iex> b = Source.Ex.from_string(\":b\", \"b.exs\")\n    iex> {:ok, project} = Rewrite.from_sources([a, b])\n    iex> {:ok, project} = Rewrite.update(project, \"a.exs\", Source.Ex.from_string(\":foo\", \"a.exs\"))\n    iex> project |> Rewrite.source!(\"a.exs\") |> Source.get(:content)\n    \":foo\"\n    iex> {:ok, project} = Rewrite.update(project, \"a.exs\", fn s -> Source.update(s, :content, \":baz\") end)\n    iex> project |> Rewrite.source!(\"a.exs\") |> Source.get(:content)\n    \":baz\"\n    iex> {:ok, project} = Rewrite.update(project, \"a.exs\", fn s -> Source.update(s, :path, \"c.exs\") end)\n    iex> Rewrite.paths(project)\n    [\"b.exs\", \"c.exs\"]\n    iex> Rewrite.update(project, \"no.exs\", Source.from_string(\":foo\", \"x.exs\"))\n    {:error, %Rewrite.Error{reason: :nosource, path: \"no.exs\"}}\n    iex> Rewrite.update(project, \"c.exs\", Source.from_string(\":foo\"))\n    {:error, %Rewrite.UpdateError{reason: :nopath, source: \"c.exs\"}}\n    iex> Rewrite.update(project, \"c.exs\", fn _ -> b end)\n    {:error, %Rewrite.UpdateError{reason: :overwrites, path: \"b.exs\", source: \"c.exs\"}}","ref":"Rewrite.html#update/3-examples"},{"type":"function","title":"Rewrite.update!/2","doc":"The same as `update/2` but raises a `Rewrite.Error` exception in case\nof an error.","ref":"Rewrite.html#update!/2"},{"type":"function","title":"Rewrite.update!/3","doc":"The same as `update/3` but raises a `Rewrite.Error` exception in case\nof an error.","ref":"Rewrite.html#update!/3"},{"type":"function","title":"Rewrite.updated?/1","doc":"Returns `true` if any source in the `rewrite` project returns `true` for\n`Source.updated?/1`.","ref":"Rewrite.html#updated?/1"},{"type":"function","title":"Examples - Rewrite.updated?/1","doc":"iex> {:ok, project} = Rewrite.from_sources([\n    ...>   Source.Ex.from_string(\":a\", \"a.exs\"),\n    ...>   Source.Ex.from_string(\":b\", \"b.exs\"),\n    ...>   Source.Ex.from_string(\"c\", \"c.txt\")\n    ...> ])\n    iex> Rewrite.updated?(project)\n    false\n    iex> project = Rewrite.update!(project, \"a.exs\", fn source ->\n    ...>   Source.update(source, :quoted, \":z\")\n    ...> end)\n    iex> Rewrite.updated?(project)\n    true","ref":"Rewrite.html#updated?/1-examples"},{"type":"function","title":"Rewrite.write/3","doc":"Writes a source to disk.\n\nThe function expects a path or a `%Source{}` as first argument.\n\nReturns `{:ok, rewrite}` if the file was written successful. See also\n`Source.write/2`.\n\nIf the given `source` is not part of the `rewrite` project then it is added.","ref":"Rewrite.html#write/3"},{"type":"function","title":"Rewrite.write_all/2","doc":"Writes all sources in the `rewrite` project to disk.\n\nThis function calls `Rewrite.Source.write/1` on all sources in the `rewrite`\nproject.\n\nReturns `{:ok, rewrite}` if all sources are written successful.\n\nReturns `{:error, reasons, rewrite}` where rewrite is updated for all sources\nthat are written successful. The reasons is a keyword list with the keys\n`File.posix()` or `:changed` and the affected path as value. The key\n`:changed` indicates a file that was changed sind reading.","ref":"Rewrite.html#write_all/2"},{"type":"function","title":"Options - Rewrite.write_all/2","doc":"+ `exclude` - a list paths to exclude form writting.\n+ `foece`, default: `false` - forces the writting of changed files.","ref":"Rewrite.html#write_all/2-options"},{"type":"type","title":"Rewrite.input/0","doc":"","ref":"Rewrite.html#t:input/0"},{"type":"type","title":"Rewrite.opts/0","doc":"","ref":"Rewrite.html#t:opts/0"},{"type":"type","title":"Rewrite.t/0","doc":"","ref":"Rewrite.html#t:t/0"},{"type":"type","title":"Rewrite.wildcard/0","doc":"","ref":"Rewrite.html#t:wildcard/0"},{"type":"behaviour","title":"Rewrite.Filetype","doc":"The behaviour for filetypes.\n\nAn implementation of the filetype behaviour extends a source. For an example,\nsee `Rewrite.Source.Ex`.","ref":"Rewrite.Filetype.html"},{"type":"callback","title":"Rewrite.Filetype.extensions/0","doc":"Returns a list of file type extensions for which the module is responsible.","ref":"Rewrite.Filetype.html#c:extensions/0"},{"type":"callback","title":"Rewrite.Filetype.fetch/2","doc":"Fetches the value for a specific `key` for the given `source`.\n\nIf `source` contains the given `key` then its value is returned in the shape\nof {:ok, value}. If `source` doesn't contain key, :error is returned.","ref":"Rewrite.Filetype.html#c:fetch/2"},{"type":"callback","title":"Rewrite.Filetype.fetch/3","doc":"Fetches the value for a specific `key` in a `source` for the given `version`.\n\nIf `source` contains the given `key` then its value is returned in the shape\nof {:ok, value}. If `source` doesn't contain key, :error is returned.","ref":"Rewrite.Filetype.html#c:fetch/3"},{"type":"callback","title":"Rewrite.Filetype.from_string/1","doc":"Returns a `Rewrite.Source` with a `filetype` from the given `string`.","ref":"Rewrite.Filetype.html#c:from_string/1"},{"type":"callback","title":"Rewrite.Filetype.from_string/2","doc":"Returns a `Rewrite.Source` with a `filetype` from the given `string` and `path`.","ref":"Rewrite.Filetype.html#c:from_string/2"},{"type":"callback","title":"Rewrite.Filetype.from_string/3","doc":"Returns a `Rewrite.Source` with a `filetype` form the `given`, `string` and `options`.","ref":"Rewrite.Filetype.html#c:from_string/3"},{"type":"callback","title":"Rewrite.Filetype.handle_update/2","doc":"This function is called when the content or path of the `source` is updated.\n\nReturns a `%Source{}` with an updated `filetype`.","ref":"Rewrite.Filetype.html#c:handle_update/2"},{"type":"callback","title":"Rewrite.Filetype.handle_update/3","doc":"This function is called when the `source` is updated by a `key` that is\nhandled by the current `filetype`.\n\nReturns a keyword with the keys `:content` and `:filetype` to update the\n`source`.","ref":"Rewrite.Filetype.html#c:handle_update/3"},{"type":"callback","title":"Rewrite.Filetype.read!/1","doc":"Returns a `Rewrite.Source` with a `filetype` from a file.","ref":"Rewrite.Filetype.html#c:read!/1"},{"type":"callback","title":"Rewrite.Filetype.read!/2","doc":"Returns a `Rewrite.Source` with a `filetype` from a file.","ref":"Rewrite.Filetype.html#c:read!/2"},{"type":"callback","title":"Rewrite.Filetype.undo/1","doc":"This function is called after an undo of the `source`.","ref":"Rewrite.Filetype.html#c:undo/1"},{"type":"type","title":"Rewrite.Filetype.extension/0","doc":"","ref":"Rewrite.Filetype.html#t:extension/0"},{"type":"type","title":"Rewrite.Filetype.key/0","doc":"","ref":"Rewrite.Filetype.html#t:key/0"},{"type":"type","title":"Rewrite.Filetype.opts/0","doc":"","ref":"Rewrite.Filetype.html#t:opts/0"},{"type":"type","title":"Rewrite.Filetype.t/0","doc":"","ref":"Rewrite.Filetype.html#t:t/0"},{"type":"type","title":"Rewrite.Filetype.updates/0","doc":"","ref":"Rewrite.Filetype.html#t:updates/0"},{"type":"type","title":"Rewrite.Filetype.value/0","doc":"","ref":"Rewrite.Filetype.html#t:value/0"},{"type":"module","title":"Rewrite.Source","doc":"A representation of some source in a project.\n\nThe `%Source{}` contains the `content` of the file given by `path`. The module\ncontains `update/3` to update the `path` and/or the `content`. The changes are\nrecorded in the `history` list.\n\nThe struct also holds `issues` for the source.\n\nThe different versions of `content` and `path` are available via `get/3`.\n\nA source is extensible via `filetype`, see `Rewrite.Filetype`.","ref":"Rewrite.Source.html"},{"type":"function","title":"Rewrite.Source.add_issue/2","doc":"Adds the given `issue` to the `source`.","ref":"Rewrite.Source.html#add_issue/2"},{"type":"function","title":"Rewrite.Source.add_issues/2","doc":"Adds the given `issues` to the `source`.","ref":"Rewrite.Source.html#add_issues/2"},{"type":"function","title":"Rewrite.Source.diff/2","doc":"Returns iodata showing all diffs of the given `source`.\n\nSee `Rewrite.TextDiff.format/3` for options.","ref":"Rewrite.Source.html#diff/2"},{"type":"function","title":"Examples - Rewrite.Source.diff/2","doc":"iex> code = \"\"\"\n    ...> def foo( x ) do\n    ...>   {:x,\n    ...>     x}\n    ...> end\n    ...> \"\"\"\n    iex> formatted = code |> Code.format_string!() |> IO.iodata_to_binary()\n    iex> source = Source.Ex.from_string(code)\n    iex> source |> Source.diff() |> IO.iodata_to_binary()\n    \"\"\n    iex> source\n    ...> |> Source.update(:content, formatted)\n    ...> |> Source.diff(color: false)\n    ...> |> IO.iodata_to_binary()\n    \"\"\"\n    1   - |def foo( x ) do\n    2   - |  {:x,\n    3   - |    x}\n      1 + |def foo(x) do\n      2 + |  {:x, x}\n    4 3   |end\n    5 4   |\n    \"\"\"","ref":"Rewrite.Source.html#diff/2-examples"},{"type":"function","title":"Rewrite.Source.file_changed?/1","doc":"Returns `true` if the file has been modified since it was read.\n\nIf the key `:from` does not contain `:file` the function returns `false`.","ref":"Rewrite.Source.html#file_changed?/1"},{"type":"function","title":"Examples - Rewrite.Source.file_changed?/1","doc":"iex> File.write(\"tmp/hello.txt\", \"hello\")\n    iex> source = Source.read!(\"tmp/hello.txt\")\n    iex> Source.file_changed?(source)\n    false\n    iex> File.write(\"tmp/hello.txt\", \"Hello, world!\")\n    iex> Source.file_changed?(source)\n    true\n    iex> source = Source.update(source, :path, nil)\n    iex> Source.file_changed?(source)\n    true\n    iex> File.write(\"tmp/hello.txt\", \"hello\")\n    iex> Source.file_changed?(source)\n    false\n    iex> File.rm!(\"tmp/hello.txt\")\n    iex> Source.file_changed?(source)\n    true\n\n    iex> source = Source.from_string(\"hello\")\n    iex> Source.file_changed?(source)\n    false","ref":"Rewrite.Source.html#file_changed?/1-examples"},{"type":"function","title":"Rewrite.Source.filetype/2","doc":"Sets the `filetype` for the `source`.","ref":"Rewrite.Source.html#filetype/2"},{"type":"function","title":"Rewrite.Source.from?/2","doc":"Returns true when `from` matches to value for key `:from`.","ref":"Rewrite.Source.html#from?/2"},{"type":"function","title":"Examples - Rewrite.Source.from?/2","doc":"iex> source = Source.from_string(\"hello\")\n    iex> Source.from?(source, :file)\n    false\n    iex> Source.from?(source, :string)\n    true","ref":"Rewrite.Source.html#from?/2-examples"},{"type":"function","title":"Rewrite.Source.from_string/3","doc":"Creates a new `%Source{}` from the given `string`.","ref":"Rewrite.Source.html#from_string/3"},{"type":"function","title":"Examples - Rewrite.Source.from_string/3","doc":"iex> source = Source.from_string(\"hello\")\n    iex> source.content\n    \"hello\"","ref":"Rewrite.Source.html#from_string/3-examples"},{"type":"function","title":"Rewrite.Source.get/2","doc":"Gets the value for `:content`, `:path` in `source` or a specific `key` in\n`filetype`.\n\nRaises `Rewrite.SourceKeyError` if the `key` can't be found.","ref":"Rewrite.Source.html#get/2"},{"type":"function","title":"Rewrite.Source.get/3","doc":"Gets the value for `:content`, `:path` in `source` or a specific `key` in\n`filetype` for the given `version`.\n\nRaises `Rewrite.SourceKeyError` if the `key` can't be found.","ref":"Rewrite.Source.html#get/3"},{"type":"function","title":"Examples - Rewrite.Source.get/3","doc":"iex> bar =\n    ...>   \"\"\"\n    ...>   defmodule Bar do\n    ...>      def bar, do: :bar\n    ...>   end\n    ...>   \"\"\"\n    iex> foo =\n    ...>   \"\"\"\n    ...>   defmodule Foo do\n    ...>      def foo, do: :foo\n    ...>   end\n    ...>   \"\"\"\n    iex> source = Source.Ex.from_string(bar)\n    iex> source = Source.update(source, :content, foo)\n    iex> Source.get(source, :content) == foo\n    true\n    iex> Source.get(source, :content, 2) == foo\n    true\n    iex> Source.get(source, :content, 1) == bar\n    true\n\n    iex> source =\n    ...>   \"hello\"\n    ...>   |> Source.from_string(\"some/where/hello.txt\")\n    ...>   |> Source.update(:path, \"some/where/else/hello.txt\")\n    ...> Source.get(source, :path, 1)\n    \"some/where/hello.txt\"\n    iex> Source.get(source, :path, 2)\n    \"some/where/else/hello.txt\"","ref":"Rewrite.Source.html#get/3-examples"},{"type":"function","title":"Rewrite.Source.has_issues?/2","doc":"Returns `true` if the `source` has issues for the given `version`.\n\nThe `version` argument also accepts `:actual` and `:all` to check whether the\n`source` has problems for the actual version or if there are problems at all.","ref":"Rewrite.Source.html#has_issues?/2"},{"type":"function","title":"Examples - Rewrite.Source.has_issues?/2","doc":"iex> source =\n    ...>   \"a + b\"\n    ...>   |> Source.Ex.from_string(\"some/where/plus.exs\")\n    ...>   |> Source.add_issue(%{issue: :foo})\n    ...>   |> Source.update(:path, \"some/where/else/plus.exs\")\n    ...>   |> Source.add_issue(%{issue: :bar})\n    iex> Source.has_issues?(source)\n    true\n    iex> Source.has_issues?(source, 1)\n    true\n    iex> Source.has_issues?(source, :all)\n    true\n    iex> source = Source.update(source, :content, \"a - b\")\n    iex> Source.has_issues?(source)\n    false\n    iex> Source.has_issues?(source, 2)\n    true\n    iex> Source.has_issues?(source, :all)\n    true","ref":"Rewrite.Source.html#has_issues?/2-examples"},{"type":"function","title":"Rewrite.Source.hash/1","doc":"Calculates the current hash from the given `source`.","ref":"Rewrite.Source.html#hash/1"},{"type":"function","title":"Examples - Rewrite.Source.hash/1","doc":"iex> source = Source.from_string(\"\"\"\n    ...> defmodule Foo do\n    ...>   def bar, do: :bar\n    ...> end\n    ...> \"\"\")\n    iex> Source.hash(source)\n     >","ref":"Rewrite.Source.html#hash/1-examples"},{"type":"function","title":"Rewrite.Source.issues/1","doc":"Returns all issues of the given `source`.","ref":"Rewrite.Source.html#issues/1"},{"type":"function","title":"Rewrite.Source.owner/1","doc":"Returns the owner of the given `source`.","ref":"Rewrite.Source.html#owner/1"},{"type":"function","title":"Rewrite.Source.put_private/3","doc":"Assigns a private `key` and `value` to the `source`.\n\nThis is not used or accessed by Rewrite, but is intended as private storage\nfor users or libraries that wish to store additional data about a source.","ref":"Rewrite.Source.html#put_private/3"},{"type":"function","title":"Examples - Rewrite.Source.put_private/3","doc":"iex> source =\n    ...>   \"a + b\"\n    ...>   |> Source.from_string()\n    ...>   |> Source.put_private(:origin, :example)\n    iex> source.private[:origin]\n    :example","ref":"Rewrite.Source.html#put_private/3-examples"},{"type":"function","title":"Rewrite.Source.read!/2","doc":"Creates a new `%Source{}` from the given `path`.","ref":"Rewrite.Source.html#read!/2"},{"type":"function","title":"Examples - Rewrite.Source.read!/2","doc":"iex> source = Source.read!(\"test/fixtures/source/hello.txt\")\n    iex> source.content\n    \"\"\"\n    hello\n    \"\"\"","ref":"Rewrite.Source.html#read!/2-examples"},{"type":"function","title":"Rewrite.Source.rm/1","doc":"Tries to delete the file `source`.\n\nReturns `:ok` if successful, or `{:error, reason}` if an error occurs.\n\nNote the file is deleted even if in read-only mode.","ref":"Rewrite.Source.html#rm/1"},{"type":"function","title":"Rewrite.Source.rm!/1","doc":"Same as `rm/1`, but raises a `Rewrite.SourceError` exception in case of\nfailure. Otherwise `:ok`.","ref":"Rewrite.Source.html#rm!/1"},{"type":"function","title":"Rewrite.Source.undo/2","doc":"Undoes the given `number` of changes.","ref":"Rewrite.Source.html#undo/2"},{"type":"function","title":"Examples - Rewrite.Source.undo/2","doc":"iex> a = Source.from_string(\"test-a\", \"test/foo.txt\")\n    iex> b = Source.update(a, :content, \"test-b\")\n    iex> c = Source.update(b, :path, \"test/bar.txt\")\n    iex> d = Source.update(c, :content, \"test-d\")\n    iex> d |> Source.undo() |> Source.get(:content)\n    \"test-b\"\n    iex> d |> Source.undo(1) |> Source.get(:content)\n    \"test-b\"\n    iex> d |> Source.undo(2) |> Source.get(:path)\n    \"test/foo.txt\"\n    iex> d |> Source.undo(3) |> Source.get(:content)\n    \"test-a\"\n    iex> d |> Source.undo(9) |> Source.get(:content)\n    \"test-a\"\n    iex> d |> Source.undo(9) |> Source.updated?()\n    false\n    iex> d |> Source.undo(-9) |> Source.get(:content)\n    \"test-d\"","ref":"Rewrite.Source.html#undo/2-examples"},{"type":"function","title":"Rewrite.Source.update/4","doc":"Updates the `content` or the `path` of a `source`.","ref":"Rewrite.Source.html#update/4"},{"type":"function","title":"Examples - Rewrite.Source.update/4","doc":"iex> source =\n    ...>   \"foo\"\n    ...>   |> Source.from_string()\n    ...>   |> Source.update(Example, :path, \"test/fixtures/new.exs\")\n    ...>   |> Source.update(:content, \"bar\")\n    iex> source.history\n    [{:content, Rewrite, \"foo\"}, {:path, Example, nil}]\n    iex> source.content\n    \"bar\"\n\nIf the new value is equal to the current value, no history will be added.\n\n    iex> source =\n    ...>   \"42\"\n    ...>   |> Source.from_string()\n    ...>   |> Source.update(Example, :content, \"21\")\n    ...>   |> Source.update(Example, :content, \"21\")\n    ...>   |> Source.update(Example, :content, \"21\")\n    iex> source.history\n    [{:content, Example, \"42\"}]","ref":"Rewrite.Source.html#update/4-examples"},{"type":"function","title":"Rewrite.Source.updated?/2","doc":"Returns `true` if the source was updated.\n\nThe optional argument `kind` specifies whether only `:code` changes or `:path`\nchanges are considered. Defaults to `:any`.","ref":"Rewrite.Source.html#updated?/2"},{"type":"function","title":"Examples - Rewrite.Source.updated?/2","doc":"iex> source = Source.from_string(\"foo\")\n    iex> Source.updated?(source)\n    false\n    iex> source = Source.update(source, :content, \"bar\")\n    iex> Source.updated?(source)\n    true\n    iex> Source.updated?(source, :path)\n    false\n    iex> Source.updated?(source, :content)\n    true","ref":"Rewrite.Source.html#updated?/2-examples"},{"type":"function","title":"Rewrite.Source.version/1","doc":"Returns the `version` of the given `source`. The value `1` indicates that the\nsource has no changes.","ref":"Rewrite.Source.html#version/1"},{"type":"function","title":"Rewrite.Source.write/2","doc":"Writes the source to disk.\n\nReturns `{:ok, source}` when the file was written successfully. The returned\n`source` does not include any previous changes or issues.\n\nIf there's an error, this function returns `{:error, error}` where `error`\nis a `Rewrite.SourceError`. You can raise it manually with `raise/1`.\n\nReturns `{:error, error}` with `reason: :nopath` if the current `path` is nil.\n\nReturns `{:error, error}` with `reason: :changed` if the file was changed\nsince reading. See also `file_changed?/1`. The option `force: true` forces\noverwriting a changed file.\n\nIf the source `:path` was updated then the old file will be deleted.\n\nMissing directories are created.","ref":"Rewrite.Source.html#write/2"},{"type":"function","title":"Options - Rewrite.Source.write/2","doc":"+ `:force`, default: `false` - forces the saving to overwrite changed files.\n+ `:rm`, default: `true` - prevents file deletion when set to `false`.","ref":"Rewrite.Source.html#write/2-options"},{"type":"function","title":"Examples - Rewrite.Source.write/2","doc":"iex> \":test\" |> Source.from_string() |> Source.write()\n    {:error, %SourceError{reason: :nopath, path: nil, action: :write}}\n\n    iex> path = \"tmp/foo.txt\"\n    iex> File.write(path, \"foo\")\n    iex> source = path |> Source.read!() |> Source.update(:content, \"bar\")\n    iex> Source.updated?(source)\n    true\n    iex> {:ok, source} = Source.write(source)\n    iex> File.read(path)\n    {:ok, \"bar\\n\"}\n    iex> Source.updated?(source)\n    false\n\n    iex> source = Source.from_string(\"bar\")\n    iex> Source.write(source)\n    {:error, %SourceError{reason: :nopath, path: nil, action: :write}}\n    iex> source |> Source.update(:path, \"tmp/bar.txt\") |> Source.write()\n    iex> File.read(\"tmp/bar.txt\")\n    {:ok, \"bar\\n\"}\n\n    iex> path = \"tmp/ping.txt\"\n    iex> File.write(path, \"ping\")\n    iex> source = Source.read!(path)\n    iex> new_path = \"tmp/pong.ex\"\n    iex> source = Source.update(source, :path, new_path)\n    iex> Source.write(source)\n    iex> File.exists?(path)\n    false\n    iex> File.read(new_path)\n    {:ok, \"ping\\n\"}\n\n    iex> path = \"tmp/ping.txt\"\n    iex> File.write(path, \"ping\")\n    iex> source = Source.read!(path)\n    iex> new_path = \"tmp/pong.ex\"\n    iex> source = Source.update(source, :path, new_path)\n    iex> Source.write(source, rm: false)\n    iex> File.exists?(path)\n    true\n\n    iex> path = \"tmp/ping.txt\"\n    iex> File.write(path, \"ping\")\n    iex> source = path |> Source.read!() |> Source.update(:content, \"peng\")\n    iex> File.write(path, \"pong\")\n    iex> Source.write(source)\n    {:error, %SourceError{reason: :changed, path: \"tmp/ping.txt\", action: :write}}\n    iex> {:ok, _source} = Source.write(source, force: true)","ref":"Rewrite.Source.html#write/2-examples"},{"type":"function","title":"Rewrite.Source.write!/1","doc":"Same as `write/1`, but raises a `Rewrite.SourceError` exception in case of\nfailure.","ref":"Rewrite.Source.html#write!/1"},{"type":"type","title":"Rewrite.Source.by/0","doc":"","ref":"Rewrite.Source.html#t:by/0"},{"type":"type","title":"Rewrite.Source.content/0","doc":"","ref":"Rewrite.Source.html#t:content/0"},{"type":"type","title":"Rewrite.Source.extension/0","doc":"","ref":"Rewrite.Source.html#t:extension/0"},{"type":"type","title":"Rewrite.Source.filetype/0","doc":"","ref":"Rewrite.Source.html#t:filetype/0"},{"type":"type","title":"Rewrite.Source.from/0","doc":"","ref":"Rewrite.Source.html#t:from/0"},{"type":"type","title":"Rewrite.Source.issue/0","doc":"","ref":"Rewrite.Source.html#t:issue/0"},{"type":"type","title":"Rewrite.Source.key/0","doc":"","ref":"Rewrite.Source.html#t:key/0"},{"type":"type","title":"Rewrite.Source.kind/0","doc":"","ref":"Rewrite.Source.html#t:kind/0"},{"type":"type","title":"Rewrite.Source.opts/0","doc":"","ref":"Rewrite.Source.html#t:opts/0"},{"type":"type","title":"Rewrite.Source.owner/0","doc":"","ref":"Rewrite.Source.html#t:owner/0"},{"type":"type","title":"Rewrite.Source.t/0","doc":"","ref":"Rewrite.Source.html#t:t/0"},{"type":"type","title":"Rewrite.Source.value/0","doc":"","ref":"Rewrite.Source.html#t:value/0"},{"type":"type","title":"Rewrite.Source.version/0","doc":"The `version` of a `%Source{}`. The version `1` indicates that the source has\nno changes.","ref":"Rewrite.Source.html#t:version/0"},{"type":"module","title":"Rewrite.Source.Ex","doc":"An implementation of `Rewrite.Filetye` to handle Elixir source files.\n\nThe module uses the [`sourceror`](https://github.com/doorgan/sourceror) package\nto provide an [extended AST](https://hexdocs.pm/sourceror/readme.html#sourceror-s-ast)\nrepresentation of an Elixir file.\n\n`Ex` extends the `source` by the key `:quoted`.","ref":"Rewrite.Source.Ex.html"},{"type":"module","title":"Examples - Rewrite.Source.Ex","doc":"iex> source = Source.Ex.from_string(\"Enum.reverse(list)\")\n    iex> Source.get(source, :quoted)\n    {{:., [trailing_comments: [], line: 1, column: 5],\n      [\n        {:__aliases__,\n         [\n           trailing_comments: [],\n           leading_comments: [],\n           last: [line: 1, column: 1],\n           line: 1,\n           column: 1\n         ], [:Enum]},\n        :reverse\n      ]},\n     [\n       trailing_comments: [],\n       leading_comments: [],\n       closing: [line: 1, column: 18],\n       line: 1,\n       column: 6\n     ], [{:list, [trailing_comments: [], leading_comments: [], line: 1, column: 14], nil}]}\n    iex> quoted = quote(do: :foo)\n    iex> source = Source.update(source, :quoted, quoted)\n    iex> Source.updated?(source)\n    true\n    iex> Source.get(source, :content)\n    \"\"\"\n    :foo\n    \"\"\"","ref":"Rewrite.Source.Ex.html#module-examples"},{"type":"function","title":"Rewrite.Source.Ex.format/2","doc":"Formats the given `source`, `code` or `quoted` into code.\n\nReturns an updated `source` when input is a `source`.\n\n    iex> code = \"\"\"\n    ...> defmodule    Foo do\n    ...>     def   foo,   do:    :foo\n    ...>    end\n    ...> \"\"\"\n    iex> Source.Ex.format(code)\n    \"\"\"\n    defmodule Foo do\n      def foo, do: :foo\n    end\n    \"\"\"\n    iex> Source.Ex.format(code, force_do_end_blocks: true)\n    \"\"\"\n    defmodule Foo do\n      def foo do\n        :foo\n      end\n    end\n    \"\"\"\n\n    iex> source = Source.Ex.from_string(\"\"\"\n    ...> defmodule    Foo do\n    ...>     def   foo,   do:    :foo\n    ...>    end\n    ...> \"\"\")\n    iex> Source.Ex.format(source, force_do_end_blocks: true)\n    \"\"\"\n    defmodule Foo do\n      def foo do\n        :foo\n      end\n    end\n    \"\"\"","ref":"Rewrite.Source.Ex.html#format/2"},{"type":"function","title":"Rewrite.Source.Ex.from_string/2","doc":"Returns a `%Rewrite.Source{}` with an added `:filetype`.","ref":"Rewrite.Source.Ex.html#from_string/2"},{"type":"function","title":"Rewrite.Source.Ex.merge_formatter_opts/2","doc":"Merges the `formatter_opts` for a `source`.","ref":"Rewrite.Source.Ex.html#merge_formatter_opts/2"},{"type":"function","title":"Rewrite.Source.Ex.modules/1","doc":"Returns the current modules for the given `source`.","ref":"Rewrite.Source.Ex.html#modules/1"},{"type":"function","title":"Rewrite.Source.Ex.modules/2","doc":"Returns the modules of a `source` for the given `version`.","ref":"Rewrite.Source.Ex.html#modules/2"},{"type":"function","title":"Examples - Rewrite.Source.Ex.modules/2","doc":"iex> bar =\n    ...>   \"\"\"\n    ...>   defmodule Bar do\n    ...>      def bar, do: :bar\n    ...>   end\n    ...>   \"\"\"\n    iex> foo =\n    ...>   \"\"\"\n    ...>   defmodule Baz.Foo do\n    ...>      def foo, do: :foo\n    ...>   end\n    ...>   \"\"\"\n    iex> source = Source.Ex.from_string(bar)\n    iex> source = Source.update(source, :content, bar   foo)\n    iex> Source.Ex.modules(source)\n    [Baz.Foo, Bar]\n    iex> Source.Ex.modules(source, 2)\n    [Baz.Foo, Bar]\n    iex> Source.Ex.modules(source, 1)\n    [Bar]","ref":"Rewrite.Source.Ex.html#modules/2-examples"},{"type":"function","title":"Rewrite.Source.Ex.put_formatter_opts/2","doc":"Puts the `formatter_opts` to the `source`.\n\nThe formatter options are in use during updating and formatting.","ref":"Rewrite.Source.Ex.html#put_formatter_opts/2"},{"type":"function","title":"Rewrite.Source.Ex.read!/2","doc":"Returns a `%Rewrite.Source{}` with an added `:filetype`.\n\nThe `content` is reading from the file under the given `path`.","ref":"Rewrite.Source.Ex.html#read!/2"},{"type":"type","title":"Rewrite.Source.Ex.t/0","doc":"","ref":"Rewrite.Source.Ex.html#t:t/0"},{"type":"module","title":"Rewrite.TextDiff","doc":"Formats the diff between two strings.","ref":"Rewrite.TextDiff.html"},{"type":"module","title":"Examples - Rewrite.TextDiff","doc":"iex> code = \"\"\"\n    ...> defmodule Foo do\n    ...>   @moduledoc   false\n    ...>\n    ...>   def foo, do:  :foo\n    ...>\n    ...>   def three_times(x) do\n    ...>     {x,\n    ...>      x,x}\n    ...>   end\n    ...>\n    ...>   def bar(x) do\n    ...>     {:bar, x}\n    ...>   end\n    ...> end\\\n    ...> \"\"\"\n    iex> formatted = code |> Code.format_string!() |> IO.iodata_to_binary()\n    iex> code\n    ...> |> Rewrite.TextDiff.format(formatted, color: false)\n    ...> |> IO.iodata_to_binary()\n    \"\"\"\n     1  1   |defmodule Foo do\n     2    - |  @moduledoc   false\n        2 + |  @moduledoc false\n     3  3   |\n     4    - |  def foo, do:  :foo\n        4 + |  def foo, do: :foo\n     5  5   |\n     6  6   |  def three_times(x) do\n     7    - |    {x,\n     8    - |     x,x}\n        7 + |    {x, x, x}\n     9  8   |  end\n    10  9   |\n         ...|\n    \"\"\"","ref":"Rewrite.TextDiff.html#module-examples"},{"type":"function","title":"Rewrite.TextDiff.default_opts/0","doc":"","ref":"Rewrite.TextDiff.html#default_opts/0"},{"type":"function","title":"Rewrite.TextDiff.format/3","doc":"Formats the diff between two strings.\n\nThe returned `iodata` shows the lines with changes and 2 lines before and\nafter the changed positions. The string contains also a gutter with line\nnumber and a `-` or `+` for removed and added lines. Multiple lines without\nchanges are marked with `...` in the gutter.","ref":"Rewrite.TextDiff.html#format/3"},{"type":"function","title":"Options - Rewrite.TextDiff.format/3","doc":"* `:after` - the count of lines printed after each change. Defaults to `2`.\n  * `:before` - the count of lines printed before each change. Defaults to `2`.\n  * `:color` - enables color in the output. Defaults to `true`.\n  * `:line_numbers` - enables line numbers. Defaults to `true`.\n  * `:line` - the line number of the first line. Defaults to `1`.\n  * `:tokenizer` - a function that splits a line of text into distinct tokens\n    that should be compared when creating a colorized diff of a single line.\n    The default tokenizer prioritizes highlighting entire words, so a line\n    that updates `two` to `three` would appear as removing `two` and adding\n    `three`, instead of keeping the `t`, removing `wo`, and adding `hree`.\n    `:tokenizer` may be a function that accepts a single argument or an MFA\n    tuple, where the line of text will be prepended to the given arguments.\n  * `:colorizer` - a function that accepts a string and a color token (`:red`,\n    `:green`, etc.) and returns iodata with that formatting applied. May be\n    a function that accepts 2 arguments or an MFA tuple, where the string and\n    color will be prepended to the given arguments. Defaults to a colorizer\n    based on `IO.ANSI.format/1`.\n  * `:format` - optional keyword list of formatting options. See \"Formatting\"\n    below.","ref":"Rewrite.TextDiff.html#format/3-options"},{"type":"function","title":"Formatting - Rewrite.TextDiff.format/3","doc":"Alternative formatting options can be passed to control the gutter, colors,\nand the separator between the gutter and line of text in the rendered diff.\nThe separator is the same for all lines, but the gutter and colors differ\ndepending on the operation: `:eq`, `:del`, `:ins`, `:skip`.\n\nThe options (and their defaults) are:\n\n  * `:separator` - `\"|\"`\n  * `:gutter`\n    * `:eq` - `\"   \"`\n    * `:del` - `\" - \"`\n    * `:ins` - `\" + \"`\n    * `:skip` - `\"...\"`\n  * `:colors`\n    * `:del` - `[text: :red, space: :red_background]`\n    * `:ins` - `[text: :green, space: :green_background]`\n    * `:skip` - `[text: :yellow]`\n    * `:separator` - `[text: :yellow]`\n\nThese top-level formatting options will be merged into passed options. For\nexample, you could change only the `:separator` with:\n\n    format(string1, string2, format: [separator: \"~ \"])\n\nSee `IO.ANSI` for info on colors.","ref":"Rewrite.TextDiff.html#format/3-formatting"},{"type":"function","title":"Examples - Rewrite.TextDiff.format/3","doc":"iex> code = \"\"\"\n    ...> defmodule Bar do\n    ...>   @moduledoc false\n    ...>\n    ...>   bar(x, y) do\n    ...>     z = x + y\n    ...>     {x,y  , z}\n    ...>   end\n    ...>\n    ...>   bar(x, y, z) do\n    ...>     {x, y, z}\n    ...>   end\n    ...> end\\\n    ...> \"\"\"\n    iex> formatted = code |> Code.format_string!() |> IO.iodata_to_binary()\n    iex> code\n    ...> |> Rewrite.TextDiff.format(formatted, color: false)\n    ...> |> IO.iodata_to_binary()\n    \"\"\"\n         ...|\n     4  4   |  bar(x, y) do\n     5  5   |    z = x + y\n     6    - |    {x,y  , z}\n        6 + |    {x, y, z}\n     7  7   |  end\n     8  8   |\n         ...|\n    \"\"\"\n    iex> code\n    ...> |> Rewrite.TextDiff.format(formatted, color: false, after: 1, before: 1)\n    ...> |> IO.iodata_to_binary()\n    \"\"\"\n         ...|\n     5  5   |    z = x + y\n     6    - |    {x,y  , z}\n        6 + |    {x, y, z}\n     7  7   |  end\n         ...|\n    \"\"\"\n    iex> code\n    ...> |> Rewrite.TextDiff.format(formatted, color: false, line_numbers: false)\n    ...> |> IO.iodata_to_binary()\n    \"\"\"\n    ...|\n       |  bar(x, y) do\n       |    z = x + y\n     - |    {x,y  , z}\n     + |    {x, y, z}\n       |  end\n       |\n    ...|\n    \"\"\"","ref":"Rewrite.TextDiff.html#format/3-examples"},{"type":"exception","title":"Rewrite.Error","doc":"An exception for when a function can not handle a source.","ref":"Rewrite.Error.html"},{"type":"type","title":"Rewrite.Error.reason/0","doc":"","ref":"Rewrite.Error.html#t:reason/0"},{"type":"type","title":"Rewrite.Error.t/0","doc":"","ref":"Rewrite.Error.html#t:t/0"},{"type":"exception","title":"Rewrite.SourceError","doc":"An exception for when a function can not handle a source.","ref":"Rewrite.SourceError.html"},{"type":"function","title":"Rewrite.SourceError.format/1","doc":"","ref":"Rewrite.SourceError.html#format/1"},{"type":"type","title":"Rewrite.SourceError.action/0","doc":"","ref":"Rewrite.SourceError.html#t:action/0"},{"type":"type","title":"Rewrite.SourceError.path/0","doc":"","ref":"Rewrite.SourceError.html#t:path/0"},{"type":"type","title":"Rewrite.SourceError.reason/0","doc":"","ref":"Rewrite.SourceError.html#t:reason/0"},{"type":"type","title":"Rewrite.SourceError.t/0","doc":"","ref":"Rewrite.SourceError.html#t:t/0"},{"type":"exception","title":"Rewrite.SourceKeyError","doc":"An exception for when a key can't be found in a source.","ref":"Rewrite.SourceKeyError.html"},{"type":"exception","title":"Rewrite.UpdateError","doc":"An exception for when a function can not handle a source.","ref":"Rewrite.UpdateError.html"},{"type":"type","title":"Rewrite.UpdateError.reason/0","doc":"","ref":"Rewrite.UpdateError.html#t:reason/0"},{"type":"type","title":"Rewrite.UpdateError.t/0","doc":"","ref":"Rewrite.UpdateError.html#t:t/0"}],"content_type":"text/markdown"}